/** A multiline text area in which user has to enter program's output */
function TextOut() {
    this.node = document.createElement('label');
    this.node.className = 'cout';
    ScoTools.createHtml('span', this.node, null, 'Textausgabe:');
    ScoTools.createHtml(false, this.node, null, ' ');
    ScoTools.createHtml('textarea', this.node);
}

/** Content of text area */
Object.defineProperty(TextOut.prototype, 'value', {
    enumerable: true,
    get() { return this.node.lastChild.value; },
    set(str) { this.node.lastChild.value = str; }
});

/** Display input? */
Object.defineProperty(TextOut.prototype, 'display', {
    enumerable: true,
    get() { return (this.node.style.display != 'none'); },
    set(doShow) { this.node.style.display = doShow ? 'block' : 'none'; }
});

/**
* Focus text input
* @return undefined
*/
TextOut.prototype.focus = function() {
    this.node.lastChild.focus();
}

/**
* Select difference of text content to `target`
* @return Object        null or object describing difference like { start: "index of first char in actual text beeing wrong", end: "index of first char in actual text beeing right (after wrong chars)", actual: "text to be replaced", target: "text to replace", isEnd: "true if there is no correct char at the end, false otherwise" }
*/
TextOut.prototype.selectDiff = function(
    target          // Wanted text content
) {
    const actual = this.value;
    if (actual == target)
        return null;
    let diffStart = 0;
    while (diffStart < actual.length && diffStart < target.length && actual[diffStart] === target[diffStart])
        ++diffStart;
    /*if (actual.length < target.length) {
        this.node.lastChild.setSelectionRange(diffStart, actual.length);
        return { start: diffStart, end: actual.length, actual: actual.substring(diffStart), target: target.substring(diffStart), isEnd: true };
    }*/
    const minEnd = diffStart + Math.max(0, actual.length - target.length);
    let diffEnd = actual.length - 1;
    while (diffEnd >= minEnd && diffEnd >= 0 && actual[diffEnd] === target[diffEnd + target.length - actual.length])
        --diffEnd;
    this.node.lastChild.setSelectionRange(diffStart, diffEnd + 1);
    return { start: diffStart, end: diffEnd + 1, actual: actual.substring(diffStart, diffEnd + 1), target: target.substring(diffStart, diffEnd + 1 + target.length - actual.length), isEnd: diffEnd == actual.length - 1 };
}
